/*
 *    NzdlWrapper.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

import java.util.Map;
import java.util.Set;
import java.util.List;


/**
 *  A Wrapper around an NzdlService object that adds functionality. 
 *
 * Typically one or more of the methods will be overridden to 
 * add that functionality
 *
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @version $Revision: 3099 $
 */
abstract public class NzdlWrapper
  extends java.lang.Object 
  implements NzdlService {

  private NzdlService service = null;
  /** 
   * The bean constructor ...
   */
  public NzdlWrapper() {
    this.service = null;
  }
  /** 
   * The normal constructor
   */
  public NzdlWrapper(NzdlService service) {
    this.service = service;
  }

  /**
   * Allow the client to configure a CORBA server.
   * @param key the parameter to be set.
   * @param values what that parameter will be set to.
   */
  public void configure( String _key, Set _values ) {
    service.configure(_key, _values);
  }

  /**
   * Obtain the set of collection names for a Greenstone library.
   * @return A set of collection names in string format.
   */
  public Set getCollectionSet( ) {
    return service.getCollectionSet();
  }

  /**
   * Check if ORB knows of this collection.
   * @param name the name of a collecion to be searched for.
   * @return boolean <TT>true</TT> if collection is found, else <TT>false</TT>.
   */
  public boolean hasCollection( String _name ) {
    return service.hasCollection(_name);
  }


  /**
   * Check if ORB can communicate with of this collection.
   * @param name the name of a collecion to be pinged.
   * @return boolean <TT>true</TT> if collection returned ping, else
   * <TT>false</TT>.
   */
  public boolean pingCollection( String _name ) {
    return service.pingCollection(_name);
  }

  /**
   * Extract information on this collection into a
   * {@link NzdlCollectionInfo NzdlCollectionInfo} object
   * @param name the collection to be retrieved.
   * @return an instance of
   * {@link NzdlCollectionInfo NzdlCollectionInfo} containing data on a
   * collection
   */
  public NzdlCollectionInfo getCollectionInfo( String _name ) {
    return service.getCollectionInfo(_name);
  }

  /**
   * Returns the set of filter options for a particular filter
   * @param name the collection name
   * @param filter the filter name
   * @return an instance of
   * {@link NzdlFilterOptions NzdlFilterOptions} containing the filter
   * options for a filter
   */
    public NzdlFilterOptions getFilterOptionSet(String _name, String _filter){
	return service.getFilterOptionSet(_name, _filter);
    }

  /**
   * Returns the set of filters for the service object.
   * @param name the collection name
   * @return a set of strings that name the filters, for example
   * BrowseFilter, QueryFilter and NullFilter.
   */
  public Set getFilterSet( String _name ) {
    return service.getFilterSet(_name);
  }

  /**
   * @param name the collection name
   * @return the Set of (name, value) value pairs for all metadata 
   * associated with a collection  
   */
  public Set getMetaTagSet( String _name ) {
    return service.getMetaTagSet(_name);
  }

  /**
   * Returns the Greenstone Markup Language (GML) text for a particular
   * document from a collection.
   * @param name the name of the collection.
   * @param docID the document identity string
   * @return the GML document in the form of a string.
   */
  public String getDocument( String _name, String _docID ) {
    return service.getDocument(_name, _docID);
  }

  /**
   * Returns the set of value(s) for a metatag from a particular document.
   * @param name collection name
   * @param docID the document identifier.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The set of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Set getMetaData( String _name, String _docID, String _metaTag ) {
    return service.getMetaData( _name, _docID, _metaTag );
  }

  /**
   * Returns a map of value(s) for a metatag from a list of documents.
   * @param name collection name
   * @param docIDs a list of document identifier strings.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The map of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Map getMetaData( String _name, List _docIDs, String _metaTag ) {
    return service.getMetaData( _name, _docIDs, _metaTag );
  }

  /**
   * Services a {@link NzdlRequest NzdlRequest} to a collection and places
   * the results into a {@link NzdlResponse NzdlResponse} object. <br>
   * Pre:   Created instances of NzdlRequest and NzdlResponse. <br>
   * Post: Placed the result data into NzdlResponse object.
   * @param name the collection name.
   * @param request an instance of NzdlRequest with query data to be serviced
   * @param response an instance of NzdlResponse. where the results of
   * sevciving the query will be placed
   */
  public void service( String _name, NzdlRequest _request, 
		       NzdlResponse _response ) {    
    service.service(_name,_request,_response);
  }
  
}
